/* -*- c++ -*-
    kmmimeparttree.cpp A MIME part tree viwer.

    This file is part of KMail, the KDE mail client.
    Copyright (c) 2002-2004 Klarälvdalens Datakonsult AB

    KMail is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    KMail is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the TQt library by Trolltech AS, Norway (or with modified versions
    of TQt that use the same license as TQt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    TQt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/



// -*- c++ -*-
#ifndef KMMIMEPARTTREE_H
#define KMMIMEPARTTREE_H

#include <tdelistview.h>
#include <tdeio/global.h>

#include <tqstring.h>

class partNode;
class KMReaderWin;
class KMMimePartTreeItem;

class KMMimePartTree : public TDEListView
{
  Q_OBJECT
  
  friend class ::KMReaderWin;

public:
  KMMimePartTree( KMReaderWin* readerWin,
                  TQWidget* parent,
                  const char* name = 0 );
  virtual ~KMMimePartTree();

  void correctSize( TQListViewItem * item );

protected slots:
  void itemClicked( TQListViewItem* );
  void itemRightClicked( TQListViewItem*, const TQPoint& );
  void slotSaveAs();
  void slotSaveAsEncoded();
  void slotSaveAll();
  void slotDelete();
  void slotEdit();
  void slotOpen();
  void slotOpenWith();
  void slotView();
  void slotProperties();
  void slotCopy();

protected:
  /** reimplemented in order to update the frame width in case of a changed
      GUI style */
  void styleChange( TQStyle& oldStyle );

  /** Set the width of the frame to a reasonable value for the current GUI
      style */
  void setStyleDependantFrameWidth();

  void saveSelectedBodyParts( bool encoded );
  void restoreLayoutIfPresent();

  /* reimpl */
  void startDrag();

  void startHandleAttachmentCommand( int type );

protected:
  KMReaderWin* mReaderWin;
  KMMimePartTreeItem* mCurrentContextMenuItem;
  int mSizeColumn;
};

class KMMimePartTreeItem :public TQListViewItem
{
public:
  KMMimePartTreeItem( KMMimePartTree * parent,
                      partNode* node,
                      const TQString & labelDescr,
                      const TQString & labelCntType  = TQString(),
                      const TQString & labelEncoding = TQString(),
                      TDEIO::filesize_t size=0 );
  KMMimePartTreeItem( KMMimePartTreeItem * parent,
                      partNode* node,
                      const TQString & labelDescr,
                      const TQString & labelCntType  = TQString(),
                      const TQString & labelEncoding = TQString(),
                      TDEIO::filesize_t size=0,
                      bool revertOrder = false );
  partNode* node() const { return mPartNode; }

  TDEIO::filesize_t origSize() const { return mOrigSize; }
  void setOrigSize( TDEIO::filesize_t size ) { mOrigSize = size; }

private:
  void setIconAndTextForType( const TQString & mimetype );

  partNode* mPartNode;
  TDEIO::filesize_t mOrigSize;
};

#endif // KMMIMEPARTTREE_H
